@@ -160,6 +160,23 @@ class TestCanonicalize(unittest.TestCase):
 
         self.assertEqual(actual, expected)
 
+    def test_v5_with_json(self):
+        hashed_body = hashlib.sha512(JSON_STRING.encode('utf-8')).hexdigest()
+        headers = {"X-Duo-Header-1": "header_value_1"}
+        expected = (
+                'Tue, 17 Nov 2020 14:12:00\n'
+                'POST\n'
+                'foo.bar52.com\n'
+                '/Foo/BaR2/qux\n\n' + hashed_body
+                +'\n630b4bfe7e9abd03da2eee8f0a5d4e60a254ec880a839bcc2223bb5b9443e8ef24d58f0'
+                 '254f1f5934bf8c017ebd0fd5b1acf86766bdbe74185e712a4092df3ed')
+        params = {}
+        body = duo_client.client.Client.canon_json(JSON_BODY)
+        actual = duo_client.client.canonicalize(
+            'POST', 'foO.BaR52.cOm', '/Foo/BaR2/qux', params, 'Tue, 17 Nov 2020 14:12:00',
+            sig_version=5, body=body, additional_headers=headers)
+        self.assertEqual(actual, expected)
+
     def test_invalid_signature_version_raises(self):
         params = duo_client.client.Client.canon_json(JSON_BODY)
         with self.assertRaises(ValueError) as e:
@@ -170,6 +187,28 @@ class TestCanonicalize(unittest.TestCase):
             e.exception.args[0],
             "Unknown signature version: {}".format(999))
 
+    def test_signature_v5_lowers_and_then_sorts_headers(self):
+        hashed_body = hashlib.sha512(JSON_STRING.encode('utf-8')).hexdigest()
+        headers = {
+            "x-duo-A": "header_value_1",
+            "X-Duo-B": "header_value_2"
+
+        }
+        expected = (
+                'Tue, 17 Nov 2020 14:12:00\n'
+                'POST\n'
+                'foo.bar52.com\n'
+                '/Foo/BaR2/qux\n\n'
+                + hashed_body
+                +'\n60be11a30e0756f2ee2afdce1db849b987dcf86c1133394b'
+                'd7bbbc9877920330c4d78aceacbb377ab8cbd9a8efe6a410fed4047376635ac71226ab46ca10d2b1')
+        params = {}
+        body = duo_client.client.Client.canon_json(JSON_BODY)
+        actual = duo_client.client.canonicalize(
+            'POST', 'foO.BaR52.cOm', '/Foo/BaR2/qux', params, 'Tue, 17 Nov 2020 14:12:00',
+            sig_version=5, body=body, additional_headers=headers)
+        self.assertEqual(actual, expected)
+
 
 class TestNormalizePageArgs(unittest.TestCase):
 
@@ -261,7 +300,7 @@ class TestSign(unittest.TestCase):
             skey='gtdfxv9YgVBYcF6dl2Eq17KUQJN2PLM2ODVTkvoT',
             **test
         )
-        expected = "0508065035a03b2a1de2f453e629e791d180329e157f65df6b3e0f08299d4321e1c5c7a7c7ee6b9e5fc80d1fb6fbf3ad5eb7c44dd3b3985a02c37aca53ec3698"
+        expected = '0508065035a03b2a1de2f453e629e791d180329e157f65df6b3e0f08299d4321e1c5c7a7c7ee6b9e5fc80d1fb6fbf3ad5eb7c44dd3b3985a02c37aca53ec3698'
         expected = ikey + ':' + expected
         if isinstance(expected, six.text_type):
             expected = expected.encode('utf-8')
@@ -375,6 +414,48 @@ class TestPaging(unittest.TestCase):
         self.assertListEqual(expected, list(response))
         self.assertEqual(1, self.client.counter)
 
+class TestAlternatePaging(unittest.TestCase):
+    def setUp(self):
+        self.client = util.CountingClient(
+            'test_ikey', 'test_akey', 'example.com', paging_limit=100)
+        self.objects = [util.MockJsonObject() for i in range(1000)]
+        self.client._connect = lambda: util.MockAlternatePagingHTTPConnection(self.objects)
+
+    def test_get_objects_paging(self):
+        response = self.client.json_cursor_api_call(
+            'GET', '/admin/v1/objects', {},
+            lambda response: response['data']
+        )
+        self.assertEqual(len(self.objects), len(list(response)))
+        self.assertEqual(10, self.client.counter)
+
+    def test_get_no_objects_paging(self):
+        self.objects = []
+        self.client._connect = lambda: util.MockAlternatePagingHTTPConnection(self.objects)
+        response = self.client.json_cursor_api_call(
+            'GET', '/admin/v1/objects', {},
+            lambda response: response['data']
+        )
+        self.assertEqual(len(self.objects), len(list(response)))
+        self.assertEqual(1, self.client.counter)
+
+    def test_get_objects_paging_limit(self):
+        response = self.client.json_cursor_api_call(
+            'GET', '/admin/v1/objects', {'limit':'250'},
+            lambda response: response['data']
+        )
+        self.assertEqual(len(self.objects), len(list(response)))
+        self.assertEqual(4, self.client.counter)
+
+    def test_get_all_objects(self):
+        response = self.client.json_cursor_api_call(
+            'GET', '/admin/v1/objects', {'limit':'1000'},
+            lambda response: response['data']
+        )
+        expected = [obj.to_json() for obj in self.objects]
+        self.assertListEqual(expected, list(response))
+        self.assertEqual(1, self.client.counter)
+
 class TestRequestsV4(unittest.TestCase):
     # usful args for testing GETs
     args_in = {
@@ -547,6 +628,38 @@ class TestParseJsonResponseAndMetadata(unittest.TestCase):
         self.assertEqual(e.exception.reason, api_res.reason)
         self.assertEqual(e.exception.data, response)
 
+    def test_response_is_a_string(self):
+        """
+        Some API requests return just a string in their response.
+        We want to make sure we handle those correctly
+        """
+        api_res = self.APIResponse(200, 'Fake reason')
+        response = {
+            'response': "just a string",
+            'stat': 'OK'
+        }
+        try:
+            self.client.parse_json_response_and_metadata(api_res, json.dumps(response))
+        except Exception:
+            self.fail("parsing raised exception for string response")
+
+    def test_response_is_a_list(self):
+        """
+        Some API requests return just a list in their response. with
+        metadata included at the top level.
+        We want to make sure we handle those correctly
+        """
+        api_res = self.APIResponse(200, "Fake reason")
+        expected_metadata = { "offset": 4 }
+        expected_response = ["multiple", "elements"]
+        response = {
+            "response": expected_response,
+            "metadata": expected_metadata,
+            "stat": "OK"
+        }
+        response, metadata = self.client.parse_json_response_and_metadata(api_res, json.dumps(response))
+        self.assertEqual(metadata, expected_metadata)
+
     def test_response_stat_isnot_OK(self):
         api_res = self.APIResponse(200, 'Fake reason')
 
