@@ -123,7 +123,28 @@ class MockPagingHTTPConnection(MockHTTPConnection):
         params = six.moves.urllib.parse.parse_qs(parsed.query)
 
         self.limit = int(params['limit'][0])
-        self.offset = int(params['offset'][0])
+
+        # offset is always present with list-based paging but cannot be
+        # present on the initial request with cursor-based paging
+        self.offset = int(params.get('offset', [0])[0])
+
+class MockAlternatePagingHTTPConnection(MockPagingHTTPConnection):
+    def read(self):
+        metadata = {}
+        metadata['total_objects'] = len(self.objects)
+        if self.offset + self.limit < len(self.objects):
+            metadata['next_offset'] = self.offset + self.limit
+        if self.offset > 0:
+            metadata['prev_offset'] = max(self.offset-self.limit, 0)
+
+        return json.dumps(
+                {"stat":"OK",
+                 "response": {
+                    "data" : self.objects[self.offset: self.offset+self.limit],
+                    "metadata": metadata
+                  },
+                },
+                cls=MockObjectJsonEncoder)
 
 
 class MockMultipleRequestHTTPConnection(MockHTTPConnection):
